@@ -141,7 +141,6 @@ solve_network(void /*node *stnlist*/)
 	 POS(stn,0) = (real)0.0;
 	 POS(stn,1) = (real)0.0;
 	 POS(stn,2) = (real)0.0;
-	 fix(stn);
       }
    }
 
@@ -649,7 +648,6 @@ replace_travs(void)
 	       mulsd(&e, &leg->v, &sc);
 	       adddd(&POSD(stn3), &POSD(stn3), &e);
 	    }
-	    fix(stn3);
 	 }
 
 	 if (!(leg->l.reverse & (FLAG_REPLACEMENTLEG | FLAG_FAKE))) {
@@ -682,7 +680,7 @@ replace_travs(void)
 	     * (not equate at start of traverse) */
 #ifndef BLUNDER_DETECTION
 	    if (fhErrStat && !fArtic) {
-	       if (!stn1->name->ident) {
+	       if (!prefix_ident(stn1->name)) {
 		  /* FIXME: not ideal */
 		  fputs("<fixed point>", fhErrStat);
 	       } else {
@@ -690,7 +688,7 @@ replace_travs(void)
 	       }
 	       fputs(fEquate ? szLinkEq : szLink, fhErrStat);
 	       if (reached_end) {
-		  if (!stn3->name->ident) {
+		  if (!prefix_ident(stn3->name)) {
 		     /* FIXME: not ideal */
 		     fputs("<fixed point>", fhErrStat);
 		  } else {
@@ -839,7 +837,6 @@ replace_trailing_travs(void)
 #endif
 	 }
 
-	 fix(stn2);
 	 add_stn_to_list(&stnlist, stn2);
 	 if (!(leg->l.reverse & (FLAG_REPLACEMENTLEG | FLAG_FAKE))) {
 	     if (TSTBIT(leg->l.flags, FLAGS_SURFACE)) {
@@ -918,7 +915,7 @@ skip_nosurvey:
 	    const char * label = NULL;
 	    if (TSTBIT(sf, SFLAGS_ANON)) {
 	       label = "";
-	    } else if (stn1->name->ident) {
+	    } else if (prefix_ident(stn1->name)) {
 	       label = sprint_prefix(stn1->name);
 	    }
 	    if (label) {
@@ -974,7 +971,7 @@ skip_nosurvey:
 	    unused_fixed_point = true;
 	 } else if (stn1->leg[0]) {
 	    prefix *pfx = stn1->leg[0]->l.to->name;
-	    if (!pfx->ident && !TSTBIT(pfx->sflags, SFLAGS_ANON)) {
+	    if (!prefix_ident(pfx) && !TSTBIT(pfx->sflags, SFLAGS_ANON)) {
 	       /* Unused fixed point with error estimates */
 	       unused_fixed_point = true;
 	    }
@@ -990,7 +987,7 @@ skip_nosurvey:
       /* For stations fixed with error estimates, we need to ignore the leg to
        * the "real" fixed point in the node stats.
        */
-      if (stn1->leg[0] && !stn1->leg[0]->l.to->name->ident &&
+      if (stn1->leg[0] && !prefix_ident(stn1->leg[0]->l.to->name) &&
 	  !TSTBIT(stn1->leg[0]->l.to->name->sflags, SFLAGS_ANON))
 	 stn1->name->shape--;
 
